Attribute VB_Name = "modUtils"
Option Explicit

Public conData As Connection 'General Connection
Public conReport As Connection 'Reports Connection

'Function to establish a connection with the database
Public Function ConnectBackend() As Boolean
On Error GoTo Handler
    Set conData = New Connection
    Set conReport = New Connection
    
    'Open a standard connection for Records
    With conData
        .Mode = adModeShareDenyNone 'Share Database
        .Provider = "Microsoft.Jet.OLEDB.4.0" 'For Access 97/2000
        .ConnectionString = "Data Source=" & App.Path & "\Database\ReportDB.mdb" 'Database Location
        .Open
    End With
    
    'Open a DATA Shape Connection For Reports
    With conReport
        .Mode = adModeShareDenyNone
        .Provider = "MSDataShape" 'Microsoft Provider for Data Shaping
        .ConnectionString = "Data Provider = Microsoft.Jet.OLEDB.4.0;" & _
                            "Data Source = " & App.Path & "\Database\ReportDB.mdb"
        .Open
    End With
    
    ConnectBackend = True
Exit Function
Handler:
    ConnectBackend = False
End Function

'Function to check whether there is data in a recordset
Public Function GetRowsOK(ByVal rst As Recordset) As Boolean
On Error GoTo Handler
    Dim varData
    
    rst.MoveFirst
    varData = rst.GetRows(1)
    GetRowsOK = True
Exit Function
Handler:
    GetRowsOK = False
End Function

'Function to Retrive a specific recordset based on a query
Public Function DLookup(ByVal SQL As String, ByVal Table As String, rst As Recordset, Optional ByVal cond As String) As Boolean
On Error GoTo Handler

    'Check whether connection exists
    If (conData.State And adStateOpen) > 0 Then
        Set rst = New Recordset
        With rst
            .ActiveConnection = conData
            .LockType = adLockReadOnly
            .CursorType = adOpenStatic
            .Open "SELECT " & SQL & " FROM " & Table & IIf(Len(cond) > 0, " WHERE " & cond, "")
        End With
        DLookup = True
    End If
    
Exit Function
Handler:
    DLookup = False
End Function

'Function to Retrive Hierarchical Recordset based on a query
Public Function DLookupEx(ByVal SQL As String, rst As Recordset) As Boolean
On Error GoTo Handler

    'Check whether the connection is open
    
    If (conReport.State And adStateOpen) > 0 Then
        Set rst = New Recordset
        With rst
            .ActiveConnection = conReport
            .LockType = adLockReadOnly
            .CursorType = adOpenStatic
            .Open SQL
        End With
        DLookupEx = True
    End If
    
Exit Function
Handler:
    Debug.Print Err.Description
    DLookupEx = False
End Function

'Function to retrive a single field based on a SQL query
Public Function LookField(ByVal SQL As String, ByVal Table As String, Optional ByVal cond As String, Optional Els)
On Error GoTo Handler
    Dim rst As Recordset
    
    If DLookup(SQL, Table, rst, cond) Then
        If GetRowsOK(rst) Then
            rst.MoveFirst
            LookField = rst.Fields(0)
        Else
            'IsMissing Function checks whether value is passed with a paramter of a function.
            'This function works with varient parameters only.
            If Not IsMissing(Els) Then LookField = Els
        End If
    Else
        If Not IsMissing(Els) Then LookField = Els
    End If
    
Exit Function
Handler:
    If Not IsMissing(Els) Then LookField = Els
    Exit Function
End Function

'This sub-function Fills a List View control with data from database
'Parameters
'lst -> The List View Control
'SQL -> SQL Statement that gets data from the databases
'Order -> Define which columns come before others

Public Sub FillList(lst As Object, SQL As String, Optional Order As String)
On Error GoTo Handler
    Dim rst As Recordset
    Dim ite As Object
    Dim ct As Integer
        
        lst.ListItems.Clear 'Clearing existing items
        Set rst = New Recordset
    
        With rst
            .LockType = adLockReadOnly
            .CursorType = adOpenStatic
            .ActiveConnection = conData
            .Open SQL
        End With
    
        If GetRowsOK(rst) Then
            rst.MoveFirst
                'Building proper order for fields display
              If Order = "" Then
                For ct = 0 To rst.Fields.Count - 1
                    Order = Order & ct & ":"
                Next
            End If
    
            While Not rst.EOF
                Set ite = lst.ListItems.Add(, , rst(CInt(GetTag(Order))))
                For ct = 1 To rst.Fields.Count - 1
                    With ite
                        On Error Resume Next
                        .SubItems(ct) = rst(CInt(GetTag(Order, ct)))
                        On Error GoTo Handler
                    End With
                Next
                rst.MoveNext
            Wend
        End If
        On Error Resume Next
        rst.Close
        Set rst = Nothing
Exit Sub

Handler:
        Resume Next
End Sub

'=======================================================
'This function fills a combo box
Public Sub FillCombo(cmb As Object, SQL As String, widths As String, Optional Order As String = "")
On Error GoTo Handler
    Dim rst As Recordset
    Dim Fieldcount As Integer, tmp As Integer

        cmb.Clear
        Set rst = New Recordset
        With rst
            .CursorType = adOpenStatic
            .LockType = adLockReadOnly
            .ActiveConnection = conData
            .Open SQL
        End With

        If GetRowsOK(rst) Then
            rst.MoveFirst
            Fieldcount = rst.Fields.Count
            cmb.ColumnCount = Fieldcount
            cmb.ColumnWidths = widths
    
            If Order = "" Then
                For tmp = 0 To rst.Fields.Count - 1
                    Order = Order & tmp & ":"
                Next
            End If

            While Not rst.EOF
                cmb.AddItem rst(CInt(GetTag(Order, 0)))
                For tmp = 1 To Fieldcount - 1
                    With cmb
                        .Column(tmp, .ListCount - 1) = rst(CInt(GetTag(Order, tmp)))
                    End With
                Next
                rst.MoveNext
            Wend
       End If
       
       On Error Resume Next
       rst.Close
       Set rst = Nothing
    Exit Sub

Handler:
    Debug.Print Err.Description
    Resume Next
End Sub

'-----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
'|GetTag Function
'|This function Retrives Specific Data from tags of controls separated by
'|specific separators such as ; : ~ etc.
'|Standard position for tag Data :
'|ParentTag(Yes/No):Can new Value Be added:Can be Modified (Yes/No):Allow Null Values(Yes/No):Error message is improper Data entered:DataBase Table Field:(Type Any-0:Numeric-1:Text Only-2:Currency-3:Percent-4):Greater than Zero (Yes:No:Yes+:Yes-)
'------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
Public Function GetTag(tag As String, Optional position As Integer = 0, Optional separator As String = ":") As String
On Error GoTo Handler
Dim i As Integer, value As String, start As Integer

start = 0

If Not tag = "" Then
    For i = 1 To position
        start = InStr(start + 1, tag, separator, vbTextCompare)
    Next
    
    GetTag = Mid(tag, start + 1, InStr(start + 1, tag, separator, vbTextCompare) - (start + 1))
Else
    GetTag = ""
End If
Exit Function

Handler:
    GetTag = ""
End Function

